# -*- coding: utf-8 -*-
# vim:fenc=utf-8


import os


DEFAULT_CONFIG = {'algorithm': {}, 'manager': {}}
MODULES_PATH = os.environ.get('MODULES_PATH', 'insights')

LOG_FORMAT = (u'{record.extra["ip"]} [{record.time:%m-%d %H:%M}]'
              '{record.level_name}::{record.channel} - {record.message}')

DEFAULT_LOG_DESTINATION = os.path.expanduser('~/.intuition/logs')
LOG_DESTINATION = DEFAULT_LOG_DESTINATION \
    if os.path.exists(DEFAULT_LOG_DESTINATION) else '/tmp'
DEFAULT_FILE_LOG = 'intuition.log'



#
# Copyright 2013 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import abc
import logbook
import json

import intuition.data.remote as remote
#FIXME Insights is an optional depency !!
#import insights.plugins.mobile as mobile


class PortfolioFactory():
    '''
    Manages portfolio during simulation, and stays aware of the situation
    through the update() method.

    User strategies call it with a dictionnnary of detected opportunities (i.e.
    buy or sell signals).  Then the optimize function computes assets
    allocation, returning a dictionnary of symbols with their weigths or amount
    to reallocate.
                                  __________________________      _____________
    signals {'google': 745.5} --> |                         | -> |            |
                                  | trade_signals_handler() |    | optimize() |
    orders  {'google': 34}    <-- |_________________________| <- |____________|

    In addition, portfolio objects can be saved in database and reloaded later,
    and user on-the-fly orders are catched and executed in remote mode. Finally
    portfolios are connected to the server broker and, if requested, send state
    messages to client.

    This is abstract class, inheretid class will eventally overwrite optmize()
    to expose their own asset allocation strategy.
    '''

    __metaclass__ = abc.ABCMeta

    # Zipline portfolio object, updated during simulation with self.date
    portfolio = None
    perfs = None
    date = None
    bullet = None

    #TODO Add in the constructor or setup parameters some general settings like
    #     maximum weights, positions, frequency, ...
    #TODO Better to return 0 stocks to trade: remove the field
    #NOTE Regarding portfolio constraints: from a set of user-defined
    # parameters, a unique set should be constructed. Then the solution
    # provided by optimize function would have to be a subset of it. (classic
    # mathematical solution) Finally it should be defined how to handle
    # non-correct solutions
    def __init__(self, configuration):
        '''
        Parameters
            configuration : dict
                Named parameters used either for general portfolio settings
                (server and constraints), and for user optimizer function
        '''
        # Portfolio owner, mainly used for database saving and client
        # communication
        self.log = logbook.Logger('intuition.portfolio')

        # Other parameters are used in user's optimize() method
        self._optimizer_parameters = configuration

        # Make the manager talk to connected clients
        self.connected = configuration.get('connected', False)
        # Send android notifications when orders are processed
        #device = configuration.get('device', '')
        #if device:
            #self.bullet = mobile.AndroidPush(device)

        # Run the server if the engine didn't, while it is asked for
        if 'server' in configuration and self.connected:
            # Getting server object instanciated anyway before
            self.server = configuration.pop('server')

        # In case user optimization would need to retrieve more data
        self.data = remote.Data()
        self.initialize(configuration)

    def initialize(self, configuration):
        ''' Users should overwrite this method '''
        pass

    @abc.abstractmethod
    def optimize(self, date, to_buy, to_sell, parameters):
        ''' Users should overwrite this method '''
        pass

    def update(self, portfolio, date, perfs=None):
        '''
        Actualizes the portfolio universe
        ________________________________
        Parameters
            portfolio: zipline.Portfolio
                ndict object storing portfolio values at the given date
            date: datetime.datetime
                Current date in zipline simulation
        '''
        # Make the manager aware of current simulation
        self.portfolio = portfolio
        self.perfs = perfs
        self.date = date

    def trade_signals_handler(self, signals, extras={}):
        '''
        Process buy and sell signals from the simulation
        ___________________________________________________________
        Parameters
            signals: dict
                hold stocks of interest, format: {"google": 0.8, "apple": -0.2}
                If the value is negative -> sell signal, otherwize buy one
                Values are ranged between [-1 1] regarding signal confidence
            extras: whatever
                Object sent from algorithm for certain managers
        ___________________________________________________________
        Return
            dict order_book, like {"google": 34, "apple": -56}
        '''

        self._optimizer_parameters['algo'] = extras
        order_book = {}

        # If value < 0, it's a sell signal on the key, else buy signal
        to_buy = dict(filter(lambda (sid, strength):
                             strength > 0, signals.iteritems()))
        to_sell = dict(filter(lambda (sid, strength):
                              strength < 0, signals.iteritems()))
        #to_buy    = [t for t in signals if signals[t] > 0]
        #NOTE With this line we can't go short
        #to_sell   = set(self.portfolio.positions.keys()).intersection(
                #[t for t in signals if signals[t] < 0])

        if not to_buy and not to_sell:
            # Nothing to do
            return {}

        # Compute the optimal portfolio allocation, using user defined function
        alloc, e_ret, e_risk = self.optimize(
            self.date, to_buy, to_sell, self._optimizer_parameters)

        #TODO Check about selling within available money
        #     and handle 250 stocks limit
        #TODO Handle max_* as well, ! already actif stocks

        ## Building orders for zipline
        #NOTE The follonwing in a separate function that could be used when
        #     catching message from user
        for t in alloc:
            ## Handle allocation returned as number of stocks to order
            if isinstance(alloc[t], int):
                order_book[t] = alloc[t]

            ## Handle allocation returned as stock weights to order
            elif isinstance(alloc[t], float):
                # Sell orders
                if alloc[t] <= 0:
                    order_book[t] = int(alloc[t] *
                                        self.portfolio.positions[t].amount)
                ## Buy orders
                else:
                    # If we already trade this ticker, substract owned amount
                    # before computing number of stock to buy
                    if self.portfolio.positions[t].amount:
                        price = self.portfolio.positions[t].last_sale_price
                    else:
                        price = signals[t]
                    order_book[t] = (int(alloc[t] *
                                     self.portfolio.portfolio_value / price)
                                     - self.portfolio.positions[t].amount)

        '''
        if self.bullet and order_book:
            # Alert user of the orders about to be processed
            # Ok... kind of fancy method
            ords = {'-1': 'You should sell', '1': 'You should buy'}
            items = ['{} {} stocks of {}'.format(
                ords[str(amount / abs(amount))], amount, ticker)
                for ticker, amount in order_book.iteritems()]
            payload = {
                'title': 'Portfolio manager notification',
                'items': items,
            }
            req = self.bullet.push(payload)
            self.log.debug(req)
        '''

        return order_book

    def advise(self, **kwargs):
        '''
        General parameters or user settings
        (maw_weigth, max_assets, max_frequency, commission cost)
        ________________________________________________________
        Parameters
            parameters: dict
                Arbitrary values to change general constraints,
                or for user algorithm settings
        '''
        for name, value in kwargs.iteritems():
            self._optimizer_parameters[name] = value

    def catch_messages(self, timeout=1):
        '''
        Listen for user messages,
        process usual orders
        '''
        msg = self.server.noblock_recv(timeout=timeout, json=True)
        #TODO msg is a command or an information, process it
        if msg:
            self.log.info('Got message from user: {}'.format(msg))
            try:
                msg = json.loads(msg)
            except:
                msg = ''
                self.log.error('Unable to parse user message')
        return msg



#
# Copyright 2013 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import time
import datetime as dt
import pytz
import calendar
from dateutil.parser import parse

import pandas as pd
from pandas.core.datetools import BMonthEnd

#import babel.numbers
#import decimal


#TODO Handle in-day dates, with hours and minutes
def normalize_date_format(date):
    '''
    Dates can be defined in many ways, but zipline use
    aware datetime objects only. Plus, the software work
    with utc timezone so we convert it.
    __________________________________________________
    Parameters
        date: str
            String date, see dateutils module for precisions
    __________________________________________________
    Return
        datetime.datetime utc tz aware object
    '''
    assert isinstance(date, str) or isinstance(date, unicode)
    local_tz = pytz.timezone(_detect_timezone())
    local_dt = local_tz.localize(parse(date), is_dst=None)
    return local_dt.astimezone(pytz.utc)

    #locale_date = parse(date)
    #if locale_date.tzinfo is None:
        #locale_date = locale_date.replace(
            #tzinfo=pytz.timezone(_detect_timezone()))
    ##FIXME astimezone() retieve 8 minutes from Paris timezone Oo 20 from
    ##      Amsterdam WTF
    #return locale_date.astimezone(pytz.utc)


def _detect_timezone():
    '''
    Experimental and temporary (since there is a world module)
    get timezone as set by the system
    '''
    import locale
    locale_code = locale.getdefaultlocale()[0]
    #return str(pytz.country_timezones[locale_code[:2]][0])
    return str(pytz.country_timezones[locale_code[-2:]][0])


def build_date_index(start='', end='', freq='D'):
    #TODO Only 'D' for backtest and '1min' for live are supported
    now = dt.datetime.now(tz=pytz.utc)
    if not start:
        if not end:
            # Live trading until the end of the day
            start = now
            end = now.replace(hour=23, minute=59, second=0)
            freq = '1min'
        else:
            end = normalize_date_format(end)
            if end > now:
                # Live trading from now to end
                start = now
                freq = '1min'
            else:
                # Backtest for a year
                start = end - 360 * pd.datetools.day
    else:
        # Only backtests support start information
        start = normalize_date_format(start)
        if not end:
            # Backtest for a year or until now
            end = start + 360 * pd.datetools.day
            if end > now:
                end = now
        else:
            end = normalize_date_format(end)
            assert start < end
    return pd.date_range(start, end, freq=freq)


def UTCdateToEpoch(utc_date):
    return calendar.timegm(utc_date.timetuple())


def dateToEpoch(local_date):
    return time.mktime(local_date.timetuple())


def epochToDate(epoch, tz=pytz.utc):
    tm = time.gmtime(epoch)
    return(dt.datetime(tm.tm_year, tm.tm_mon, tm.tm_mday,
           tm.tm_hour, tm.tm_min, tm.tm_sec, 0, tz))


def dateFormat(epoch):
    ''' Convert seconds of epoch time to date POSIXct format %Y-%M-%D %H:%m '''
    return time.strftime("%Y-%m-%d %H:%M", time.localtime(epoch))


def toMonthly(frame, how):
    #TODO: generic function ?
    offset = BMonthEnd()
    return frame.groupby(offset.rollforward).aggregate(how)


def getOffset(values):
    #Inspect pandas to figure out a much better method
    ''' Transform delta, Serie, Dataframe and dates into Offset '''
    assert (len(values) > 0)
    if isinstance(values, dt.timedelta):
        delta = values
    elif isinstance(values, pd.DataFrame) or isinstance(values, pd.Series):
        delta = values.index[1] - values.index[0]
    elif isinstance(values[0], dt.datetime):
        assert (len(values) > 1)
        delta = values[1] - values[0]
    else:
        raise ValueError()
    assert(delta.days != 0 or delta.seconds != 0)
    rest = delta.days % 30
    if delta.seconds > 0 and delta.days == 0:
        offset = pd.datetools.Minute(delta.seconds * 60)
    elif delta.days < 5:
        offset = pd.datetools.BDay(delta.days)
    elif rest < 5 or rest > 25:
        offset = pd.datetools.BMonthEnd(int(round(float(delta.days) / 30.0)))
    else:
        offset = pd.datetools.BDay(delta.days)
    #NOTE useful ?
    #return pd.DateOffset(days=delta.days, minutes=delta.seconds*60)
    return offset


def dateToIndex(dates, tz=pytz.utc):
    assert (len(dates) > 1)
    assert (isinstance(dates[0], dt.datetime))
    index = pd.date_range(dates[0], dates[-1], freq=getOffset(dates))
    if not dates[0].tzinfo:
        return index.tz_localize(tz)
    return index



""" yahoofinance

This module provides a Python API for retrieving stock data from Yahoo Finance.

sample usage:
>>> import yahoofinance
>>> print yahoofinance.get_price('GOOG')
529.46
"""


import urllib
from bs4 import BeautifulSoup


def __request(symbol, stat):
    ''' Yahoo request factory method '''
    url = 'http://finance.yahoo.com/d/quotes.csv?s=%s&f=%s' % (symbol, stat)
    return urllib.urlopen(url).read().strip().strip('"')


def get_all(symbol):
    """
    Get all available quote data for the given ticker symbol.

    Returns a dictionary.
    """
    values = __request(symbol, 'l1c1va2xj1b4j4dyekjm3m4rr5p5p6s7').split(',')
    data = {}
    data['price'] = values[0]
    data['change'] = values[1]
    data['volume'] = values[2]
    data['avg_daily_volume'] = values[3]
    data['stock_exchange'] = values[4]
    data['market_cap'] = values[5]
    data['book_value'] = values[6]
    data['ebitda'] = values[7]
    data['dividend_per_share'] = values[8]
    data['dividend_yield'] = values[9]
    data['earnings_per_share'] = values[10]
    data['52_week_high'] = values[11]
    data['52_week_low'] = values[12]
    data['50day_moving_avg'] = values[13]
    data['200day_moving_avg'] = values[14]
    data['price_earnings_ratio'] = values[15]
    data['price_earnings_growth_ratio'] = values[16]
    data['price_sales_ratio'] = values[17]
    data['price_book_ratio'] = values[18]
    data['short_ratio'] = values[19]
    return data


def get_price(symbol):
    return __request(symbol, 'l1')


def get_change(symbol):
    return __request(symbol, 'c1')


def get_volume(symbol):
    return __request(symbol, 'v')


def get_avg_daily_volume(symbol):
    return __request(symbol, 'a2')


def get_stock_exchange(symbol):
    name = __request(symbol, 'x')
    if name == 'NasdaqNM':
        name = 'NASDAQ'
    return name


def get_market_cap(symbol):
    return __request(symbol, 'j1')


def get_book_value(symbol):
    return __request(symbol, 'b4')


def get_ebitda(symbol):
    return __request(symbol, 'j4')


def get_dividend_per_share(symbol):
    return __request(symbol, 'd')


def get_dividend_yield(symbol):
    return __request(symbol, 'y')


def get_earnings_per_share(symbol):
    return __request(symbol, 'e')


def get_52_week_high(symbol):
    return __request(symbol, 'k')


def get_52_week_low(symbol):
    return __request(symbol, 'j')


def get_50day_moving_avg(symbol):
    return __request(symbol, 'm3')


def get_200day_moving_avg(symbol):
    return __request(symbol, 'm4')


def get_price_earnings_ratio(symbol):
    return __request(symbol, 'r')


def get_price_earnings_growth_ratio(symbol):
    return __request(symbol, 'r5')


def get_price_sales_ratio(symbol):
    return __request(symbol, 'p5')


def get_price_book_ratio(symbol):
    return __request(symbol, 'p6')


def get_short_ratio(symbol):
    return __request(symbol, 's7')


def get_name(symbol):
    return __request(symbol, 'n')


def get_sector(symbol):
    '''
    Uses BeautifulSoup to scrape stock sector from Yahoo! Finance website
    '''
    url = 'http://finance.yahoo.com/q/pr?s=%s+Profile' % symbol
    soup = BeautifulSoup(urllib.urlopen(url).read())
    try:
        sector = soup.find('td', text='Sector:').\
            find_next_sibling().string.encode('utf-8')
    except:
        sector = ''
    return sector


def get_industry(symbol):
    '''
    Uses BeautifulSoup to scrape stock industry from Yahoo! Finance website
    '''
    url = 'http://finance.yahoo.com/q/pr?s=%s+Profile' % symbol
    soup = BeautifulSoup(urllib.urlopen(url).read())
    try:
        industry = soup.find('td', text='Industry:').\
            find_next_sibling().string.encode('utf-8')
    except:
        industry = ''
    return industry


def get_type(symbol):
    '''
    Uses BeautifulSoup to scrape symbol category from Yahoo! Finance website
    '''
    url = 'http://finance.yahoo.com/q/pr?s=%s+Profile' % symbol
    soup = BeautifulSoup(urllib.urlopen(url).read())
    if soup.find('span', text='Business Summary'):
        return 'Stock'
    elif soup.find('span', text='Fund Summary'):
        asset_type = 'Fund'
    elif symbol.find('^') == 0:
        asset_type = 'Index'
    else:
        pass
    return asset_type


def get_indices(symbol):
    url = 'http://finance.yahoo.com/q/pr?s=%s+Profile' % symbol
    soup = BeautifulSoup(urllib.urlopen(url).read())
    indices = []
    try:
        tmp_idx = soup.find('td', text='Index Membership:').\
            find_next_sibling().findAll('a')
        for idx in tmp_idx:
            indices.append(idx.text)
    except:
        pass
    return ','.join(indices)


def get_historical_prices(symbol, start_date, end_date):
    """
    Get historical prices for the given ticker symbol.
    Dates may either be a date object or a string with the following format:
    'YYYYMMDD'

    Returns a nested list.
    """
    if isinstance(start_date, str):
        # Months are zero-based
        start_m = str(int(start_date[4:6]) - 1)
        start_d = str(int(start_date[6:8]))
        start_y = str(int(start_date[0:4]))
    else:
        # Months are zero-based
        start_m = str(start_date.month - 1)
        start_d = str(start_date.day)
        start_y = str(start_date.year)

    if isinstance(end_date, str):
        # Months are zero-based
        end_m = str(int(end_date[4:6]) - 1)
        end_d = str(int(end_date[6:8]))
        end_y = str(int(end_date[0:4]))
    else:
        # Months are zero-based
        end_m = str(end_date.month - 1)
        end_d = str(end_date.day)
        end_y = str(end_date.year)

    url = 'http://ichart.yahoo.com/table.csv?s=%s&' % symbol + \
          'd=%s&' % end_m + \
          'e=%s&' % end_d + \
          'f=%s&' % end_y + \
          'g=d&' + \
          'a=%s&' % start_m + \
          'b=%s&' % start_d + \
          'c=%s&' % start_y + \
          'ignore=.csv'
    days = urllib.urlopen(url).readlines()
    data = [day[:-2].split(',') for day in days]
    return data



#
# Copyright 2013 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# -*- coding: utf-8 -*-
# vim:fenc=utf-8


import pandas as pd
import time
import pytz
import datetime
import abc

from zipline.sources.data_source import DataSource
from zipline.gens.utils import hash_args

from intuition.data.utils import smart_selector


def _build_event(date, sid, series):
    ''' Format the input to a zipline compliant event '''
    event = {
        'dt': date,
        'sid': sid}
    if isinstance(series, pd.Series):
        if sid in series.keys():
            event.update({'price': series[sid],
                          'volume': 1000})
        else:
            event.update(series.to_dict())
    elif isinstance(series, pd.DataFrame):
        event.update(series[sid].to_dict())

    return event


class DataFactory(DataSource):
    '''
    Intuition surcharge of DataSource zipline class

    Configuration options:

    sids   : list of values representing simulated internal sids
             It can be an explicit list of symbols, or a universe like nyse,20
             (that will pick up 20 random symbols from nyse exchange)
    start  : start date
    delta  : timedelta between internal events
    filter : filter to remove the sids
    '''

    __metaclass__ = abc.ABCMeta

    def __init__(self, data_descriptor, **kwargs):
        assert isinstance(data_descriptor['index'],
                          pd.tseries.index.DatetimeIndex)

        #self.data_descriptor = data_descriptor
        # Unpack config dictionary with default values.
        self.sids = smart_selector(kwargs.get('sids',
                                              data_descriptor['universe']))
        self.start = kwargs.get('start', data_descriptor['index'][0])
        self.end = kwargs.get('end', data_descriptor['index'][-1])
        self.index = data_descriptor['index']

        # Hash_value for downstream sorting.
        self.arg_string = hash_args(data_descriptor, **kwargs)

        # Check provided informations
        assert isinstance(self.sids, list)

        self._raw_data = None
        self.initialize(data_descriptor, **kwargs)

    def initialize(self, data_descriptor, **kwargs):
        ''' Abstract method for user custom initialization'''
        pass

    @property
    def instance_hash(self):
        return self.arg_string

    @property
    def raw_data(self):
        if not self._raw_data:
            self._raw_data = self.raw_data_gen()
        return self._raw_data

    @abc.abstractmethod
    def get_data(self):
        ''' Users should overwrite this method '''
        pass

    def raw_data_gen(self):
        self.data = self.get_data()

        if isinstance(self.data, pd.DataFrame):
            for date, series in self.data.iterrows():
                for sid in self.sids:
                    yield _build_event(date, sid, series)

        elif isinstance(self.data, pd.Panel):
            for date in self.data.major_axis:
                df = self.data.major_xs(date)
                for sid, series in df.iterkv():
                    yield _build_event(date, sid, series)

        else:
            raise TypeError("Invalid data source type")


class LiveDataFactory(DataFactory):
    '''
    Surcharge of DataFactory for live stream sources
    '''

    __metaclass__ = abc.ABCMeta

    wait_interval = 15

    def _wait_for_dt(self, date):
        '''
        Only return when we reach given datetime
        '''
        # Intuition works with utc dates, conversion
        # are made for I/O
        now = datetime.datetime.now(pytz.utc)
        while now < date:
            print('Waiting for {} / {}'.format(now, date))
            time.sleep(self.wait_interval)
            now = datetime.datetime.now(pytz.utc)

    def _get_updated_index(self):
        '''
        truncate past dates in index
        '''
        late_index = self.index
        current_dt = datetime.datetime.now(pytz.utc)
        selector = (late_index.day > current_dt.day) \
            | ((late_index.day == current_dt.day)
                & (late_index.hour > current_dt.hour)) \
            | ((late_index.day == current_dt.day)
                & (late_index.hour == current_dt.hour)
                & (late_index.minute >= current_dt.minute))

        return self.index[selector]

    def raw_data_gen(self):
        index = self._get_updated_index()
        for date in index:
            self._wait_for_dt(date)
            snapshot = self.get_data()

            for sid, series in snapshot.iterkv():
                yield _build_event(date, sid, series)



#
# Copyright 2013 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import pytz
import pandas as pd
import logbook

from zipline.finance.trading import TradingEnvironment
from zipline.utils.factory import create_simulation_parameters

import intuition.constants as constants
from intuition.data.utils import Exchanges
from intuition.data.loader import LiveBenchmark
from intuition.core.analyzes import Analyze
import intuition.utils.utils as utils


log = logbook.Logger('intuition.core.engine')


def _intuition_module(location):
    ''' Build the module path and import it '''
    path = location.split('.')
    obj = path.pop(-1)
    return utils.dynamic_import('.'.join(path), obj)


#NOTE Is there still a point to use here a constructor object instead of a
#     simple function ?
class TradingEngine(object):
    ''' Factory class wrapping zipline Backtester, returns the requested algo
    ready for use '''

    def __new__(self, identity, modules,
                strategy_conf=constants.DEFAULT_CONFIG):

        algo_obj = _intuition_module(modules['algorithm'])
        algo_obj.identity = identity
        trading_algo = algo_obj(properties=strategy_conf['algorithm'])

        trading_algo.set_logger(logbook.Logger('algo.' + identity))

        if modules['data']:
            trading_algo.set_data_generator(_intuition_module(modules['data']))

        # Use a portfolio manager
        if modules['manager']:
            log.info('initializing manager {}'.format(modules['manager']))
            # Linking to the algorithm the configured portfolio manager
            trading_algo.manager = _intuition_module(modules['manager'])(
                strategy_conf['manager'])
        else:
            trading_algo.manager = None
            log.info('no portfolio manager used')

        return trading_algo


class Simulation(object):
    ''' Take a trading strategy and evalute its results '''

    def __init__(self, configuration):
        self.configuration = configuration
        self.context = None

    def configure(self):
        '''
        Prepare dates, data, trading environment for simulation
        '''
        last_trade = self.configuration['index'][-1]
        if last_trade > pd.datetime.now(pytz.utc):
            # This is live trading
            self.set_benchmark_loader(LiveBenchmark(
                last_trade, frequency='minute').surcharge_market_data)
        else:
            self.set_benchmark_loader(None)

        self.context = self._configure_context(self.configuration['exchange'])

    def set_benchmark_loader(self, load_function):
        ''' Define a custom benchmark loader for zipline to use '''
        self.load_market_data = load_function

    def _configure_context(self, exchange=''):
        '''
        Setup from exchange traded on benchmarks used, location
        and method to load data market while simulating
        '''
        # Environment configuration
        if exchange in Exchanges:
            trading_context = TradingEnvironment(
                bm_symbol=Exchanges[exchange]['symbol'],
                exchange_tz=Exchanges[exchange]['timezone'],
                load=self.load_market_data)
        else:
            raise NotImplementedError('Because of computation limitation, \
                trading worldwide not permitted currently')

        return trading_context

    def run(self, identity, data, strategy):
        ''' Wrapper of zipline run() method. Use the configuration set so far
        to build up the trading environment and launch the system '''
        engine = TradingEngine(identity,
                               self.configuration['modules'],
                               strategy)

        #NOTE This method does not change anything
        #engine.set_sources([DataLiveSource(data_tmp)])
        #TODO A new command line parameter ? only minutely and daily
        #     (and hourly normaly) Use filter parameter of datasource ?
        #engine.set_data_frequency(self.configuration['frequency'])
        engine.is_live = self.configuration['live']

        # Running simulation with it
        #FIXME crash if trading one day that is not a trading day
        with self.context:
            sim_params = create_simulation_parameters(
                capital_base=strategy['manager']['cash'],
                start=self.configuration['index'][0],
                end=self.configuration['index'][-1])

            daily_stats = engine.trade(data, sim_params=sim_params)

        return Analyze(
            results=daily_stats,
            metrics=engine.risk_report,
            configuration=self.configuration)



#
# Copyright 2013 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import argparse
import logbook

import intuition.constants as constants
import intuition.utils.utils as utils


log = logbook.Logger('intuition.core.configuration')


def parse_commandline():
    log.debug('parsing commandline arguments')

    parser = argparse.ArgumentParser(
        description='Intuition, the terrific trading system')
    parser.add_argument('-V', '--version',
                        action='version',
                        version='%(prog)s v0.1.3 Licence Apache 2.0',
                        help='Print program version')
    parser.add_argument('-v', '--showlog',
                        action='store_true',
                        help='Print logs on stdout')
    parser.add_argument('-c', '--context',
                        action='store', default='file::conf.yaml',
                        help='Provides the way to build context')
    parser.add_argument('-i', '--id',
                        action='store', default='gekko',
                        help='Customize the session id')
    args = parser.parse_args()

    return args.id, args.context, args.showlog


def context(driver):
    driver = driver.split('::')
    builder_name = '{}.contexts.{}'.format(constants.MODULES_PATH, driver[0])

    build_context = utils.dynamic_import(builder_name, 'build_context')
    if not build_context:
        return {}, {'algorithm': {}, 'manager': {}}

    log.info('building context')
    return build_context(driver[1])



'''
Tests for intuition.core.configuration
'''

import unittest
import intuition.core.configuration as configuration


class ConfigurationTestCase(unittest.TestCase):

    def test_fail_context_load(self):
        conf, strategy = configuration.context('unavailable')
        self.assertTrue('algorithm' in strategy and 'manager' in strategy)
        self.assertFalse(conf)
        self.assertFalse(strategy['algorithm'])
        self.assertFalse(strategy['manager'])



#!/usr/bin/python
# encoding: utf-8
#
# Copyright 2013 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from random import shuffle
import os
import pandas as pd


#TODO This is quick and dirty
def detect_exchange(universe):
    if not isinstance(universe, list):
        universe = universe.split(',')
    if universe[0] in Exchanges:
        exchange = universe[0]
    else:
        if universe[0].find('/') > 0:
            exchange = 'forex'
        elif universe[0].find('.pa') > 0:
            exchange = 'paris'
        else:
            exchange = 'nasdaq'
    return exchange


def market_sids_list(exchange, n=-1):
    csv_file = '{}.csv'.format(
        os.path.join(os.environ['HOME'], '.intuition/data', exchange.lower()))
    df = pd.read_csv(csv_file)
    sids_list = df['Symbol'].tolist()
    shuffle(sids_list)
    if n > 0:
        sids_list = sids_list[:n]
    return sids_list


def smart_selector(sids):
    if not isinstance(sids, list):
        sids = sids.split(',')
    if sids[0] in Exchanges:
        if len(sids) == 2:
            n = int(sids[1])
        else:
            n = -1
        sids = market_sids_list(sids[0], n)

    return sids


#TODO Complete with :
# http://en.wikipedia.org/wiki/List_of_stock_exchange_opening_times
# http://en.wikipedia.org/wiki/List_of_S&P_500_companies
def filter_market_hours(dates, exchange):
    ''' Only return market open hours from UTC timezone'''
    #NOTE UTC times ! Beware of summer and winter hours
    if dates.freq >= pd.datetools.Day():
        # Daily or lower frequency, no hours filter required
        return dates
    if exchange == 'paris':
        selector = ((dates.hour > 6) & (dates.hour < 16)) | \
            ((dates.hour == 17) & (dates.minute < 31))
    elif exchange == 'london':
        selector = ((dates.hour > 8) & (dates.hour < 16)) | \
            ((dates.hour == 16) & (dates.minute > 31))
    elif exchange == 'tokyo':
        selector = ((dates.hour > 0) & (dates.hour < 6))
    elif exchange == 'nasdaq' or exchange == 'nyse':
        selector = ((dates.hour > 13) & (dates.hour < 21)) | \
            ((dates.hour == 13) & (dates.minute > 31))
    else:
        # Forex or Unknown market, return as is
        return dates

    # Pandas dataframe filtering mechanism
    return dates[selector]


FX_PAIRS = ['EUR/USD', 'USD/JPY', 'GBP/USD',
            'EUR/GBP', 'USD/CHF', 'EUR/JPY',
            'EUR/CHF', 'USD/CAD', 'AUD/USD',
            'GBP/JPY', 'AUD/JPY', 'AUD/NZD',
            'CAD/JPY', 'CHF/JPY', 'NZD/USD']


# World exchanges caracteristics
Exchanges = {
    # Market code, from yahoo stock code to google market code (needed for
    # reliable download)
    # Later Londres = LON
    #FIXME Forex is temporary
    'cac40': {'symbol': '^FCHI',
              'timezone': 'Europe/Paris',
              'code': 1001,
              'google_market': 'EPA'},
    'forex': {'symbol': '^FCHI',
              'timezone': 'Europe/London',
              'indexes': [],
              'code': 1002},
    'nasdaq': {'symbol': '^GSPC',
               'timezone': 'US/Eastern',
               'code': 1003,
               'indexes': ['nasdaq', 'nyse'],
               'google_market': 'NASDAQ'},
    'nyse': {'symbol': '^GSPC',
             'timezone': 'US/Eastern',
             'code': 1004,
             'google_market': 'NYSE'}
}


class Fields:
    QUOTES = ['open', 'low', 'high', 'close', 'volume', 'adj_close']


#TODO Same for google json and xml retrieving
googleCode = dict()


yahooCode = {'ask': 'a', 'average daily volume': 'a2', 'ask size': 'a5',
             'bid': 'b', 'ask rt': 'b2', 'bid rt': 'b3', 'dividend yield': 'y',
             'book value': 'b4', 'bid size': 'b6', 'change and percent': 'c',
             'change': 'c1', 'commission': 'c3', 'change rt': 'c6',
             'after hours change rt': 'c8', 'dividend': 'd',
             'last trade date': 'd1', 'trade date': 'd2', 'earnings': 'e',
             'error': 'e1', 'eps estimate year': 'e7',
             'eps estimate next year': 'e8', 'eps estimate next quarter': 'e9',
             'float shares': 'f6', 'day low': 'g', 'day high': 'h',
             '52-week low': 'j', '52-week high': 'k',
             'holdings gain percent': 'g1', 'annualized gain': 'g3',
             'holdings gain': 'g4', 'holdings gain percent rt': 'g5',
             'holdings gain rt': 'g6', 'more info': 'i', 'order book rt': 'i5',
             'market capitalization': 'j1', 'market cap rt': 'j3',
             'EBITDA': 'j4', 'change from 52-week': 'j5',
             'percent change from 52-week low': 'j6',
             'last trade rt with time': 'k1', 'change percent rt': 'k2',
             'last trade size': 'k3', 'change from 52-week high': 'k4',
             'percent change from 52-week high': 'k5',
             'last trade with time': 'l', 'last trade price': 'l1',
             'high limit': 'l2', 'low limit': 'l3', 'day range': 'm',
             'day range rt': 'm2', '50-day ma': 'm3', '200-day ma': 'm4',
             'percent change from 50-day ma': 'm8', 'name': 'n', 'notes': 'n4',
             'open': 'o', 'previous close': 'p', 'price paid': 'p1',
             'change percent': 'p2', 'price/sales': 'p5', 'price/book': 'p6',
             'ex-dividend date': 'q', 'p/e ratio': 'r', 'dividend date': 'r1',
             'p/e ratio rt': 'r2', 'peg ratio': 'r5',
             'price/eps estimate year': 'r6',
             'price/eps estimate next year': 'r7', 'symbol': 's',
             'shares owned': 's1', 'short ratio': 's7',
             'last trade time': 't1', 'trade links': 't6',
             'ticker trend': 't7', '1 year target price': 't8', 'volume': 'v',
             'holdings value': 'v1', 'holdings value rt': 'v7',
             '52-week range': 'w', 'day value change': 'w1',
             'day value change rt': 'w4', 'stock exchange': 'x'}



#
# Copyright 2013 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# -*- coding: utf-8 -*-
# vim:fenc=utf-8


import abc

from zipline.algorithm import TradingAlgorithm
from zipline.sources import DataFrameSource
#from zipline.protocol import Event
#import zipline.protocol


class TradingFactory(TradingAlgorithm):
    '''
    Intuition surcharge of main zipline class.
    Its role is slightly different : the user will eventually expose an event()
    method meant to return buy and sell signals, processed further by the
    portfolio strategy. However it remains fully compatible with zipline method
    (i.e. it's stil possible to execute orders within the event method and not
    consider any portfolio strategy aside)
    '''

    __metaclass__ = abc.ABCMeta

    is_live = False
    #TODO Use another zipline mechanism
    day = 0
    sids = []
    middlewares = []

    def __init__(self, *args, **kwargs):
        self.data_generator = DataFrameSource
        TradingAlgorithm.__init__(self, *args, **kwargs)

    def use(self, func, when='whenever'):
        ''' Append a middleware to the algorithm '''
        #NOTE A middleware Object ?
        self.middlewares.append({
            'call': func,
            'name': func.__name__,
            'args': func.func_code.co_varnames,
            'when': when})

    def trade(self, source, sim_params=None):
        '''
        if self.is_live:
            benchmark_return_source = [
                Event({'dt': dt,
                       'returns': 0.0,
                       'type': zipline.protocol.DATASOURCE_TYPE.BENCHMARK,
                       'source_id': 'benchmarks'})
                for dt in source['index']
                if dt.date() >= sim_params.period_start.date()
                and dt.date() <= sim_params.period_end.date()]
          '''

        if isinstance(source, dict):
            source = self.data_generator(source)

        #return self.run(source, sim_params, benchmark_return_source)
        # 0.5.9 compatibility
        return self.run(source, sim_params)

    #TODO How can I use several sources ?
    def set_data_generator(self, generator_class):
        ''' Register a data source to the algorithm '''
        self.data_generator = generator_class

    #NOTE I'm not superfan of initialize + warming
    def warming(self, data):
        ''' Called at the first handle_data frame '''
        pass

    @abc.abstractmethod
    def event(self, data):
        ''' Users should overwrite this method '''
        pass

    def handle_data(self, data):
        ''' Method called for each event by zipline. In intuition this is the
        place to factorize algorithms and then call event() '''
        self.day += 1
        signals = {}

        if self.initialized:
            self.manager.update(
                self.portfolio,
                self.datetime,
                self.perf_tracker.cumulative_risk_metrics.to_dict())
        else:
            # Perf_tracker needs at least a turn to have an index
            self.warming(data)
            self.initialized = True
            self.sids = data.keys()

        signals = self.event(data)

        self._call_middlewares()

        if signals:
            order_book = self.manager.trade_signals_handler(signals)
            self.process_orders(order_book)

    def process_orders(self, order_book):
        ''' Default and costant orders processor. Overwrite it for more
        sophistiated strategies '''
        for stock in order_book:
            self.order(stock, order_book[stock])
            self.logger.debug('{}: Ordered {} {} stocks'.format(
                self.datetime, stock, order_book[stock]))

    def _call_one_middleware(self, middleware):
        ''' Evaluate arguments and execute the middleware function '''
        args = {}
        for arg in middleware['args']:
            if hasattr(self, arg):
                # same as eval() but safer for arbitrary code execution
                args[arg] = reduce(getattr, arg.split('.'), self)
        self.logger.info('calling middleware event {}'
                         .format(middleware['name']))
        middleware['call'](**args)

    def _check_condition(self, when):
        ''' Verify that the middleware condition is True '''
        #TODO Use self.datetime to evaluate <when>
        return True

    def _call_middlewares(self):
        ''' Execute the middleware stack '''
        for middleware in self.middlewares:
            if self._check_condition(middleware['when']):
                self._call_one_middleware(middleware)



#
# Copyright 2014 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import logbook
from pandas.core.datetools import BDay
import numpy as np
import math
import datetime as dt

#from intuition.utils.utils import reIndexDF


log = logbook.Logger('intuition.core.finance')


#NOTE This is temporary copied from QSTK library
#     which will be more used in the future
def qstk_get_sharpe_ratio(rets, risk_free=0.00):
    """
    @summary Returns the daily Sharpe ratio of the returns.
    @param rets: 1d numpy array or fund list of daily returns (centered on 0)
    @param risk_free: risk free returns, default is 0%
    @return Annualized rate of return, not converted to percent
    """
    f_dev = np.std(rets, axis=0)
    f_mean = np.mean(rets, axis=0)

    f_sharpe = (f_mean * 252 - risk_free) / (f_dev * np.sqrt(252))

    return f_sharpe


def moving_average(data, periods, type='simple'):
    """
    compute a <periods> period moving average.
    type is 'simple' | 'exponential'
    """
    data = np.asarray(data)
    if type == 'simple':
        weights = np.ones(periods)
    else:
        weights = np.exp(np.linspace(-1., 0., periods))

    weights /= weights.sum()

    mavg = np.convolve(data, weights, mode='full')[:len(data)]
    mavg[:periods] = mavg[periods]
    return mavg


def relative_strength(prices, periods=14):
    """
    compute the <periods> period relative strength indicator
    http://stockcharts.com/school/doku.php?\
        id=chart_school:glossary_r#relativestrengthindex
    http://www.investopedia.com/terms/r/rsi.asp
    """
    deltas = np.diff(prices)
    seed = deltas[:periods + 1]
    up = seed[seed >= 0].sum() / periods
    down = -seed[seed < 0].sum() / periods
    ratio = up / down
    rsi = np.zeros_like(prices)
    rsi[:periods] = 100. - 100. / (1. + ratio)

    for i in range(periods, len(prices)):
        delta = deltas[i - 1]  # Cause the diff is 1 shorter

        if delta > 0:
            upval = delta
            downval = 0.
        else:
            upval = 0.
            downval = -delta

        up = (up * (periods - 1) + upval) / periods
        down = (down * (periods - 1) + downval) / periods

        ratio = up / down
        rsi[i] = 100. - 100. / (1. + ratio)

    return rsi


def moving_average_convergence(data, nslow=26, nfast=12):
    """
    compute the MACD (Moving Average Convergence/Divergence)
    using a fast and slow exponential moving avg
    return value is emaslow, emafast, macd which are len(data) arrays
    """
    emaslow = moving_average(data, nslow, type='exponential')
    emafast = moving_average(data, nfast, type='exponential')
    return emaslow, emafast, emafast - emaslow


def cc_annualize_returns(ret_per_period, periods_per_year):
    return math.log(1 + annualized_returns(ret_per_period, periods_per_year))


def annualized_returns(ret_per_period, periods_per_year):
    ''' Could use:
    res1 = averageReturns(Series([ret_per_period]*periods_per_year), \
            period=1, type='net')
    '''
    return pow(1 + ret_per_period, periods_per_year) - 1


def average_returns(ts, **kwargs):
    ''' Compute geometric average returns from a returns time serie'''
    average_type = kwargs.get('type', 'net')
    if average_type == 'net':
        relative = 0
    else:
        relative = -1  # gross
    #start = kwargs.get('start', ts.index[0])
    #end = kwargs.get('end', ts.index[len(ts.index) - 1])
    #delta = kwargs.get('delta', ts.index[1] - ts.index[0])
    period = kwargs.get('period', None)
    if isinstance(period, int):
        pass
    #else:
        #ts = reIndexDF(ts, start=start, end=end, delta=delta)
        #period = 1
    avg_ret = 1
    for idx in range(len(ts.index)):
        if idx % period == 0:
            avg_ret *= (1 + ts[idx] + relative)
    return avg_ret - 1


def cc_returns(ts, **kwargs):
    start = kwargs.get('start', None)
    end = kwargs.get('end', dt.datetime.today())
    delta = kwargs.get('deltaya', BDay())
    period = kwargs.get('period', None)
    rets = returns(ts, type='net', start=start, end=end,
                   delta=delta, period=period)
    return math.log(1 + rets)


#TODO care of dividends
#TODO care of inflation
def returns(ts, **kwargs):
    '''
    Compute returns on the given period

    @param ts : time serie to process
    @param kwargs.type: gross or simple returns
    @param delta : period betweend two computed returns
    @param start : with end, will return the return betweend this elapsed time
    @param period : delta is the number of lines/periods provided
    @param end : so said
    @param cumulative: compute cumulative returns
    '''
    returns_type = kwargs.get('type', 'net')
    cumulative = kwargs.get('cumulative', False)
    if returns_type == 'net':
        relative = 0
    else:
        relative = 1  # gross
    start = kwargs.get('start', None)
    end = kwargs.get('end', dt.datetime.today())
    #delta = kwargs.get('delta', None)
    period = kwargs.get('period', 1)
    if isinstance(start, dt.datetime):
        log.debug('{} / {} -1'.format(ts[end], ts[start]))
        return ts[end] / ts[start] - 1 + relative
    #elif isinstance(delta, pd.DateOffset) or isinstance(delta, dt.timedelta):
        #FIXME timezone problem
        #FIXME reIndexDF is deprecated
        #ts = reIndexDF(ts, delta=delta)
        #period = 1
    rets_df = ts / ts.shift(period) - 1 + relative
    if cumulative:
        return rets_df.cumprod()
    return rets_df[1:]


def daily_returns(ts, **kwargs):
    ''' re-compute ts on a daily basis '''
    relative = kwargs.get('relative', 0)
    return returns(ts, delta=BDay(), relative=relative)


def panel_to_retsDF(data, kept_field='close', output='dataframe'):
    '''
    @summary transform data in DataAccess format to a dataframe
             suitable for qstk.tsutils.optimizePortfolio()
    @param data: data like quotes['close']['google']
    @output : a dataframe, cols = companies, rows = dates
    '''
    #TODO Here a need of data normalisation
    df = data[kept_field]
    df.fillna(method='ffill')
    df.fillna(method='backfill')
    if output == 'array':
        return returns(df, relative=0).values
    return returns(df, relative=0)  # 1 in doc, 0 in example


def sharpe_ratio(ts):
    #TODO: Dataframe handler
    rets = daily_returns(ts)
    return (np.mean(rets) / rets.stdev()) * np.sqrt(len(rets))


def high_low_spread(df, offset):
    ''' Compute continue spread on given datafrme every offset period '''
    #TODO: handling the offset period with reindexing or resampling, sthg like:
    # subIndex = df.index[conditions]
    # df = df.reindex(subIndex)
    return df['high'] - df['low']



#
# Copyright 2013 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# Benchmark loader modified to allow live data streaming


from datetime import datetime
import pandas as pd
from collections import OrderedDict

import zipline.data.loader as zipline

import intuition.data.utils as datautils


class LiveBenchmark(object):
    def __init__(self, end, frequency='daily', loopback=4):
        self.last_trading_day = end
        self.loopback = loopback
        if frequency == 'minute':
            self.offset = pd.datetools.Minute()
        elif frequency == 'daily':
            self.offset = pd.datetools.Day()
        else:
            raise NotImplementedError()

    def normalize_date(self, test_date):
        ''' Same function as zipline.finance.trading.py'''
        test_date = pd.Timestamp(test_date, tz='UTC')
        return pd.tseries.tools.normalize_date(test_date)

    def surcharge_market_data(self, bm_symbol='^GSPC'):
        #TODO Parametric
        #event_dt = datetime.today().replace(tzinfo=pytz.utc)
        event_dt = self.normalize_date(datetime.now())

        #TODO Handle invalid code
        for exchange, infos in datautils.Exchanges.iteritems():
            if infos['symbol'] == bm_symbol:
                code = datautils.Exchanges[exchange]['code']
                break

        bm_returns, tr_curves = zipline.load_market_data(bm_symbol)

        dates = pd.date_range(event_dt,
                              periods=len(bm_returns))
        #NOTE What is tr_curves['tid'] ?
        #TODO Replace values to detect the fake later
        tr_fake = OrderedDict(sorted(
            ((pd.Timestamp(event_dt + i*self.offset), c)
             for i, c in enumerate(tr_curves.values())),
            key=lambda t: t[0]))

        bm_fake = pd.Series([code] * len(dates), index=dates)
        for i, dt in enumerate(tr_curves.keys()):
            pd.Timestamp(event_dt + i * self.offset)

        return bm_fake, tr_fake



#
# Copyright 2013 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class IntuitionError(Exception):
    msg = None

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.message = str(self)

    def __str__(self):
        msg = self.msg.format(**self.kwargs)
        return msg

    __unicode__ = __str__
    __repr__ = __str__


# http://www.siteduzero.com/informatique/tutoriels/les-exceptions-9/\
#     liste-des-exceptions-built-in
class Example(IntuitionError):
    '''
    Temporary example
    '''
    msg = "{transform} requires {fields}. Event cannot be processed."



#
# Copyright 2014 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import requests
import logbook
import os
from pandas import DataFrame, Series


log = logbook.Logger('intuition.data.forex')


def forex_rates(user, password, pairs='', fmt='csv'):
    url = 'http://webrates.truefx.com/rates/connect.html'
    params = '?&q=ozrates&c={}&f={}&s=n'.format(','.join(pairs), fmt)
    auth = requests.get(url + params,
                        auth=(user, password))

    if auth.ok:
        log.debug('[{}] Request successful {}:{}'
                  .format(auth.headers['date'], auth.reason, auth.status_code))
    #return auth.content.split('\n')[:-2]
    return auth.content.split('\n')


def _fx_mapping(raw_response):
    ''' Map raw output to clearer labels '''
    dict_data = dict()
    for pair in raw_response:
        pair = pair.split(',')
        #FIXME Timestamp = year 45173
        dict_data[pair[0]] = {'TimeStamp': pair[1],
                              'Bid.Price': float(pair[2] + pair[3]),
                              'Ask.Price': float(pair[4] + pair[5]),
                              'High': float(pair[6]),
                              'Low': float(pair[7])}
    return dict_data


#FIXME 'Not authorized' mode works weird
class ConnectTrueFX(object):
    auth_url = ('http://webrates.truefx.com/rates/connect.html?\
        u={}&p={}&q=ozrates&c={}&f={}')
    query_url = ('http://webrates.truefx.com/rates/connect.html?\
        id={}&f={}&c={}')

    def __init__(self, credentials='', pairs=[], fmt='csv'):
        #NOTE Without authentification you still can access some data
        #FIXME Not authorized response prevent from downloading quotes.
        #      However later you indeed retrieve 10 defaults
        self._code = None
        if not credentials:
            log.info('No credentials provided, inspecting environment')
            credentials = os.environ.get('TRUEFX_API', ':')

        credentials = credentials.split(':')
        auth = requests.get(
            self.auth_url.format(credentials[0],
                                 credentials[1],
                                 ','.join(pairs), fmt))
        if auth.ok:
            log.debug('[{}] Authentification successful {}:{}'
                      .format(auth.headers['date'],
                              auth.reason,
                              auth.status_code))
            log.debug('Got: {}'.format(auth.content))

        ### Remove '\r\n'
        self._code = auth.content[:-2]

    def query_trueFX(self, pairs='', fmt='csv'):
        ''' Perform a request against truefx data '''
        if isinstance(pairs, str):
            pairs = [pairs]
        response = requests.get(
            self.query_url.format(self._code, fmt, ','.join(pairs)))

        mapped_data = _fx_mapping(response.content.split('\n')[:-2])
        if len(mapped_data) == 1:
            return Series(mapped_data)
        return DataFrame(mapped_data)

    def is_active(self):
        ''' Indicate wether the connection is still on or not '''
        return isinstance(self._code, str) and (self._code != 'not authorized')

    def __del__(self):
        #TODO Deconnection
        pass



#
# Copyright 2013 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import json
import re
import socket
from urllib2 import urlopen
#import babel.numbers
#import decimal


def dynamic_import(mod_path, obj_name):
    ''' Take a string and return the corresponding module '''
    try:
        module = __import__(mod_path, fromlist=['whatever'])
    except ImportError, description:
        print(description)
        return None

    if hasattr(module, obj_name):
        obj = getattr(module, obj_name)
    else:
        print('module {} has no attribute {}'.
              format(module.__name__, obj_name))
        return None

    return obj


def activate_pdb_hook():
    ''' Catch exceptions with a prompt for post-mortem analyzis'''
    def debug_exception(type_exception, value, tb):
        import pdb
        pdb.post_mortem(tb)

    import sys
    sys.excepthook = debug_exception


#NOTE Could use pprint module
def emphasis(obj, align=True):
    ''' Clearer data printing '''
    if isinstance(obj, dict):
        if align:
            pretty_msg = os.linesep.join(
                ["%25s: %s" % (k, obj[k]) for k in sorted(obj.keys())])
        else:
            pretty_msg = json.dumps(obj, indent=4, sort_keys=True)
    else:
        return obj
    return pretty_msg


def get_local_ip(public=False):
    ''' Utility for logbook information injection '''
    if public:
        data = str(urlopen('http://checkip.dyndns.com/').read())
        ip_addr = re.compile(
            r'Address: (\d+\.\d+\.\d+\.\d+)').search(data).group(1)
    else:
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.connect(('google.com', 0))
        ip_addr = sock.getsockname()[0]
    return ip_addr


def apply_mapping(raw_row, mapping):
    '''
    Override this to hand craft conversion of row.
    '''
    row = {target: mapping_func(raw_row[source_key])
           for target, (mapping_func, source_key)
           in mapping.fget().items()}
    return row



#
# Copyright 2013 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import sys
import logbook
from logbook.queues import ZeroMQHandler
from logbook.more import ColorizedStderrHandler

from intuition.utils.utils import get_local_ip
import intuition.constants as constants


log = logbook.Logger('intuition.default.logger')


def inject_information(record):
    ''' Add ip information to the logger output '''
    record.extra['ip'] = get_local_ip()


def get_nestedlog(level='debug',
                  show_log=False,
                  filename=constants.DEFAULT_FILE_LOG,
                  uri=None):
    ''' Intuition formated logger '''
    level = level.upper()
    # Default uri: tcp://127.0.0.1:5540
    if uri is not None:
        handlers = [ZeroMQHandler(uri)]
    else:
        handlers = [
            logbook.NullHandler(),
            #logbook.NullHandler(level=logbook.DEBUG, bubble=True),
            #ColorizedStderrHandler(format_string=log_format, level='ERROR'),
            logbook.FileHandler(
                '{}/{}'.format(constants.LOG_DESTINATION, filename),
                level=level)
            #FIXME Doesn't show anything
            #logbook.Processor(inject_information)
        ]
        if show_log:
            handlers.append(
                logbook.StreamHandler(sys.stdout,
                                      format_string=constants.LOG_FORMAT,
                                      level=level))

    return logbook.NestedSetup(handlers)


def get_colorlog():
    return logbook.NestedSetup([
        logbook.StreamHandler(sys.stdout, format_string=constants.LOG_FORMAT),
        ColorizedStderrHandler(format_string=constants.LOG_FORMAT,
                               level='NOTICE'),
        #Processor(inject_information)
    ])



#
# Copyright 2013 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import pandas as pd


#TODO Quandl symbol decorator
# Ressources : http://www.quandl.com/help/api/resources
# Or the search API : https://github.com/quandl/Python
# Seems to be {PROVIDER}/{MARKET_SUFFIX}_{GOOGLE_SYMBOL}
#NOTE with_market symbol, build jxr.pa => JXR:EPA
#TODO mega class decorator which will harmonize eveything
def use_google_symbol(fct):
    '''
    Removes ".PA" or other market indicator from yahoo symbol
    convention to suit google convention
    '''
    def decorator(symbols):
        google_symbols = []

        # If one symbol string
        if isinstance(symbols, str):
            symbols = [symbols]

        for symbol in symbols:
            dot_pos = symbol.find('.')
            google_symbols.append(
                symbol[:dot_pos] if (dot_pos > 0) else symbol)

        return fct(google_symbols)
    return decorator


def invert_dataframe_axis(fct):
    '''
    Make dataframe index column names,
    and vice et versa
    '''
    def decorator(*args, **kwargs):
        df_to_invert = fct(*args, **kwargs)
        assert isinstance(df_to_invert, pd.DataFrame)
        return pd.DataFrame(df_to_invert.to_dict().values(),
                            index=df_to_invert.to_dict().keys())
    return decorator



#
# Copyright 2013 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import pytz
import pandas as pd
import numpy as np
import logbook

from zipline.data.benchmarks import get_benchmark_returns

from finance import qstk_get_sharpe_ratio


log = logbook.Logger('intuition.core.analyze')


class Analyze():
    ''' Handle backtest results and performances measurments '''
    def __init__(self, **kwargs):
        # R analysis file only needs portfolio returns
        self.returns = kwargs.pop('returns') if 'returns' in kwargs else None

        # Final risk measurments as returned by the backtester
        self.results = kwargs.pop('results') if 'results' in kwargs else None

        # Simulation rolling performance
        self.metrics = kwargs.pop('metrics') if 'metrics' in kwargs else None

        # You better should know what was simulation's parameters
        self.configuration = kwargs.pop('configuration', None)

    def _to_perf_array(self, timestamp, key, length):
        return np.array([self.metrics[timestamp][i][key] for i in length])

    def rolling_performances(self, timestamp='one_month'):
        ''' Filters self.perfs '''
        #TODO Study the impact of month choice
        #TODO Check timestamp in an enumeration
        #TODO Implement other benchmarks for perf computation
        # (zipline issue, maybe expected)

        if self.metrics:
            perfs = {}
            length = range(len(self.metrics[timestamp]))
            index = self._get_index(self.metrics[timestamp])
            perf_keys = self.metrics['one_month'][0].keys()
            perf_keys.pop(perf_keys.index('period_label'))

            perfs['period'] = np.array(
                [pd.datetime.date(date) for date in index])
            for key in perf_keys:
                perfs[key] = self._to_perf_array(timestamp, key, length)
        else:
            #TODO Get it from DB if it exists
            raise NotImplementedError()

        return pd.DataFrame(perfs, index=index)

    def overall_metrics(self, timestamp='one_month', metrics=None):
        '''
        Use zipline results to compute some performance indicators
        '''
        perfs = dict()

        # If no rolling perfs provided, computes it
        if metrics is None:
            metrics = self.rolling_performances(timestamp=timestamp)
        riskfree = np.mean(metrics['treasury_period_return'])

        perfs['sharpe'] = qstk_get_sharpe_ratio(
            metrics['algorithm_period_return'].values, risk_free=riskfree)
        perfs['algorithm_period_return'] = (
            ((metrics['algorithm_period_return'] + 1).cumprod()) - 1)[-1]
        perfs['max_drawdown'] = max(metrics['max_drawdown'])
        perfs['algo_volatility'] = np.mean(metrics['algo_volatility'])
        perfs['beta'] = np.mean(metrics['beta'])
        perfs['alpha'] = np.mean(metrics['alpha'])
        perfs['benchmark_period_return'] = (
            ((metrics['benchmark_period_return'] + 1).cumprod()) - 1)[-1]

        return perfs

    def get_returns(self, benchmark=''):
        returns = {}

        if benchmark:
            try:
                benchmark_data = (
                    get_benchmark_returns(benchmark,
                                          self.configuration['index'][0],
                                          self.configuration['index'][-1]))
            except Exception as e:
                raise KeyError(e)
        else:
            #TODO Automatic detection given exchange market (on command line) ?
            raise NotImplementedError()

        #NOTE Could be more efficient. But len(benchmark_data.date) !=
        # len(self.results.returns.index). Maybe because of different markets
        dates = pd.DatetimeIndex([d.date for d in benchmark_data])

        returns['benchmark_return'] = pd.Series(
            [d.returns for d in benchmark_data], index=dates)
        returns['benchmark_c_return'] = (
            (returns['benchmark_return'] + 1).cumprod()) - 1
        returns['algo_return'] = pd.Series(
            self.results.returns.values, index=dates)
        returns['algo_c_return'] = pd.Series(
            ((self.results.returns.values + 1).cumprod()) - 1, index=dates)

        df = pd.DataFrame(returns, index=dates)

        if benchmark is None:
            df = df.drop(['benchmark_return', 'benchmark_c_return'], axis=1)
        return df

    def _get_index(self, perfs):
        #NOTE No frequency infos or just period number ?
        start = pytz.utc.localize(pd.datetime.strptime(
            perfs[0]['period_label'] + '-01', '%Y-%m-%d'))
        end = pytz.utc.localize(pd.datetime.strptime(
            perfs[-1]['period_label'] + '-01', '%Y-%m-%d'))
        return pd.date_range(start - pd.datetools.BDay(10),
                             end,
                             freq=pd.datetools.MonthBegin())



#!/usr/bin/env python
#
# Copyright 2013 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import sys
import os
from glob import glob

from setuptools import setup, find_packages

from intuition import __version__, __author__, __licence__


LONG_DESCRIPTION = None
README_MARKDOWN = None


def get_requirements():
    with open('./requirements.txt') as requirements:
        # Avoid github based requirements and replace them
        deps = requirements.read().split('\n')[:-3]
        deps.append('zipline>=0.5.11.dev')
        deps.append('pandas>=0.13.0.dev')
        return deps

with open('README.md') as markdown_source:
    README_MARKDOWN = markdown_source.read()

if 'upload' in sys.argv:
    # Converts the README.md file to ReST, since PyPI uses ReST for formatting,
    # This allows to have one canonical README file, being the README.md
    # The conversion only needs to be done on upload.
    # Otherwise, the pandoc import and errors that are thrown when
    # pandoc are both overhead and a source of confusion for general
    # usage/installation.
    import pandoc
    pandoc.core.PANDOC_PATH = '/usr/bin/pandoc'
    doc = pandoc.Document()
    doc.markdown = README_MARKDOWN
    LONG_DESCRIPTION = doc.rst
else:
    # If pandoc isn't installed, e.g. when downloading from pip,
    # just use the regular README.
    LONG_DESCRIPTION = README_MARKDOWN

setup(
    name='intuition',
    version=__version__,
    description='A trading system building blocks',
    author=__author__,
    author_email='xavier.bruhiere@gmail.com',
    packages=find_packages(),
    long_description=LONG_DESCRIPTION,
    license=__licence__,
    install_requires=get_requirements(),
    url="https://github.com/hackliff/intuition",
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Operating System :: OS Independent',
        'Intended Audience :: Science/Research',
        'Topic :: Office/Business :: Financial',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: System :: Distributed Computing',
    ],
    scripts=['app/intuition'],
    data_files=[(os.path.expanduser('~/.intuition/data'), glob('./data/*'))],
    dependency_links=[
        'http://github.com/pydata/pandas/tarball/master#egg=pandas-0.13.0.dev',
        'http://github.com/quantopian/zipline/tarball/master#egg=zipline-0.5.11.dev']
)



#
# Copyright 2013 Xavier Bruhiere
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import pandas as pd
import pytz
import os
import logbook

import Quandl


log = logbook.Logger('intuition.data.quandl')


def multi_codes(fct):
    '''
    Decorator that allows to use Data.Quandl.fetch with multi codes and get a
    panel of it
    '''
    def build_panel(self, codes, **kwargs):
        if isinstance(codes, str):
            # One code, do nothing special
            data = fct(self, codes, **kwargs)
        elif isinstance(codes, list):
            # Multi codes, build a panel
            tmp_data = {}
            for code in codes:
                tmp_data[code] = fct(self, code, **kwargs)
            data = pd.Panel(tmp_data)
        else:
            raise TypeError('quandl codes must be one string or a list')

        #NOTE The algorithm can't detect missing values this way...
        #NOTE An interpolate() method could be more powerful
        return data.fillna(method='pad')
    return build_panel


class DataQuandl(object):
    '''
    Quandl.com as datasource
    '''
    def __init__(self, quandl_key=''):
        self.quandl_key = quandl_key if quandl_key != '' \
            else os.environ["QUANDL_API_KEY"]

    #TODO Use of search feature for more powerfull and flexible use
    @multi_codes
    def fetch(self, code, **kwargs):
        '''
        Quandl entry point in datafeed object
        _____________________________________
        Parameters
            code: str
                quandl data code
            kwargs: dict
                keyword args passed to quandl call
        _____________________________________
        Return:
            data: pandas.dataframe or numpy array
                 returned from quandl call
        '''
        log.debug('fetching QuanDL data (%s)' % code)
        # This way you can use your credentials even if
        # you didn't provide them to the constructor
        if 'authtoken' in kwargs:
            self.quandl_key = kwargs.pop('authtoken')

        # Harmonization: Quandl call start_date trim_start
        if 'start_date' in kwargs:
            kwargs['trim_start'] = kwargs.pop('start_date')
        if 'end_date' in kwargs:
            kwargs['trim_end'] = kwargs.pop('end_date')

        try:
            data = Quandl.get(code, authtoken=self.quandl_key, **kwargs)
            data.index = data.index.tz_localize(pytz.utc)
        except:
            log.error('** Fetching %s from Quandl' % code)
            data = pd.DataFrame()
        return data
